require 'pimodel'
require 'test/unit'

class TestAgent < Test::Unit::TestCase
	
	#
	# Test bound names
	#
	def test_bound_names
	
		agent1 = Agent.new("A",['b','x'],Restriction.new('y',Input.new('b','z',
											  Summation.new(Output.new('z','x'), Output.new('z','y')))))
											  											  
		assert_equal(['y','z'],agent1.bn)
	
	end

    #
    # Test a simple substitution
    #
    def test_simple_substitution
    
        agent1 = Agent.new("N1",['i','o','e1'],Composition.new(
                           Input.new("i",Tau.new(Output.new("e1"))),
                           Input.new("e1",Tau.new(Output.new("o")))                
                           ))
                           
        assert_equal("N1(i,o,e1)=(i.t.'e1.0 | e1.t.'o.0)",agent1.to_definition)
        
        agent1.substitute(["i1","o1","e1"])
                
        assert_equal("N1(i1,o1,e1)=(i1.t.'e1.0 | e1.t.'o1.0)",agent1.to_definition)
    
    end
    
    #
    # Test if substitution respects bound names
    #
    def test_bound_substitution
    
        agent1 = Agent.new("N1",['i','o'],Restriction.new("e1",Composition.new(
                           Input.new("i",Tau.new(Output.new("e1","o"))),
                           Input.new("e1","o",Tau.new(Output.new("o"))))          
                           ))
                           
        assert_equal("N1(i,o)=(^e1)(i.t.'e1<o>.0 | e1(o).t.'o.0)",agent1.to_definition)
                        
        assert_equal(['e1','o'],agent1.bn)
        
        agent1.substitute(["i1","o1"])
                
        assert_equal("N1(i1,o1)=(^e1)(i1.t.'e1<o1>.0 | e1(o).t.'o.0)",agent1.to_definition)
    
    end
    
    #
    # Test if substitution respects bound names in hierarchical order
    #
    def test_bound_subsitution_2
    
        agent1 = Agent.new("N1",['i','o'],Restriction.new("e1",Composition.new(
                           Input.new("i",Tau.new(Output.new("e1","o"))),
                           Input.new("e1","o",Tau.new(Output.new("o"))),
                           Output.new("o")               
                           )))
                           
        assert_equal("N1(i,o)=(^e1)(i.t.'e1<o>.0 | e1(o).t.'o.0 | 'o.0)",agent1.to_definition)
        
        assert_equal(['e1','o'],agent1.bn)
        
        agent1.substitute(["i1","o1"])
                
        assert_equal("N1(i1,o1)=(^e1)(i1.t.'e1<o1>.0 | e1(o).t.'o.0 | 'o1.0)",agent1.to_definition)
    
    end

end